/*!	@file
	@brief Resource Database

	@author wakura
	@date 2008
*/
/*
	Copyright (C) 2008, wakura

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose, 
	including commercial applications, and to alter it and redistribute it 
	freely, subject to the following restrictions:

		1. The origin of this software must not be misrepresented;
		   you must not claim that you wrote the original software.
		   If you use this software in a product, an acknowledgment
		   in the product documentation would be appreciated but is
		   not required.

		2. Altered source versions must be plainly marked as such, 
		   and must not be misrepresented as being the original software.

		3. This notice may not be removed or altered from any source
		   distribution.
*/
#include "stdafx.h"
#include "CRcDb.h"

//__declspec(dllimport) LCID GetRcLang(void);

CRcDb* CRcDb::m_lpRcDbThis = NULL;

LPCTSTR CRcDb::GetString( UINT uID )
{
	RcDbStr* lpRcDbStr = FindString( uID );
	if( NULL == lpRcDbStr ) return NULL;
	return lpRcDbStr->m_lpMember[ RCDB_TYPE_DEFAULT ];
}

LPCTSTR CRcDb::GetMenuString( UINT uID, ERcDbType eRcDbType, TCHAR chShortCut )
{
	RcDbStr* lpRcDbStr = FindString( uID );
	if( NULL == lpRcDbStr ) return NULL;

	SplitString( lpRcDbStr );

	LPTSTR lp;
	switch( eRcDbType ){
	case RCDB_TYPE_MENU_ENABLE: 
		lp = lpRcDbStr->m_lpMember[ RCDB_TYPE_MENU_ENABLE ];
		break;
	case RCDB_TYPE_MENU_DISABLE:
		lp =  lpRcDbStr->m_lpMember[ RCDB_TYPE_MENU_DISABLE ];
		break;
	case RCDB_TYPE_MENU_ENABLE_SC: 
		lp = MakeShortCut( lpRcDbStr->m_lpMember[ RCDB_TYPE_MENU_ENABLE_SC ], chShortCut );
		break;
	case RCDB_TYPE_MENU_DISABLE_SC:
		lp = MakeShortCut( lpRcDbStr->m_lpMember[ RCDB_TYPE_MENU_DISABLE_SC ], chShortCut );
		break;
	case RCDB_TYPE_DEFAULT:
	default:
		lp =  lpRcDbStr->m_lpMember[ RCDB_TYPE_DEFAULT ];
		break;
	}

	return lp;
}

CRcDb::RcDbStr* CRcDb::FindString( UINT uID )
{
	MapRcDbStr::const_iterator iter = m_mapString.find( uID );
	if( iter == m_mapString.end() ){
		return CacheString( uID );
	}
	return (RcDbStr*)iter->second;
}

CRcDb::RcDbStr* CRcDb::CacheString( UINT uID )
{
	TCHAR tszString[ 1024 ];
	_tcscpy( tszString, _T("") );
	LoadString( uID, tszString, sizeof( tszString ) / sizeof( TCHAR ) );
	RcDbStr* lpRcDbStr = new RcDbStr;
	lpRcDbStr->m_lpMember[ 0 ] = new TCHAR[ _tcslen( tszString ) + 1 ];
	_tcscpy( lpRcDbStr->m_lpMember[ 0 ], tszString );
	lpRcDbStr->m_lpMember[ 1 ] = NULL;
	lpRcDbStr->m_lpMember[ 2 ] = NULL;
	lpRcDbStr->m_lpMember[ 3 ] = NULL;
	lpRcDbStr->m_lpMember[ 4 ] = NULL;
	m_mapString.insert( PairRcDb( uID, lpRcDbStr ) );
	return lpRcDbStr;
}

void CRcDb::SplitString( RcDbStr* lpRcDbStr )
{
	if( NULL != lpRcDbStr->m_lpMember[ 1 ] ) return;

	LPTSTR lpS = lpRcDbStr->m_lpMember[ 0 ];
	for( int i = 1; i < 5; i++ ){
		LPTSTR lpE = _tcsstr( lpS, _T(",") );
		if( NULL != lpE ){
			*lpE = _T('\0');
			lpRcDbStr->m_lpMember[ i ] = ++lpE;
		}else{
			lpE = lpS + _tcslen( lpS );
			lpRcDbStr->m_lpMember[ i ] = lpE;
		}
		lpS = lpE;
	}
	return;
}

LPTSTR CRcDb::MakeShortCut( LPTSTR lpStr, TCHAR chShortCut )
{
	if( 0 != chShortCut ) return lpStr;
	LPTSTR lp = _tcsstr( lpStr, _T("&?") );
	if( NULL != lp ){
		lp++;
		*lp = chShortCut;
	}
	return lpStr;
}

void CRcDb::RemoveAllString( void )
{
	RcDbStr* lpRcDbStr;
	MapRcDbStr::iterator iter = m_mapString.begin();
	for( iter = m_mapString.begin(); iter != m_mapString.end(); iter++ ){
		lpRcDbStr = (RcDbStr*)iter->second;
		iter->second = NULL;
		delete [] lpRcDbStr->m_lpMember[ 0 ];
		delete lpRcDbStr;
	}
	m_mapString.empty();
	return;
}

bool CRcDb::OpenRcDb( LCID uLangID )
{
	CloseRcDb();

	if( uLangID != 0 ){
		TCHAR tszPath[ MAX_PATH ];
		TCHAR tszName[ MAX_PATH ];
		_stprintf( tszName, RCDB_DLL_NAME, uLangID );
		HINSTANCE hRcDb = ::LoadLibrary( MakeRcDbPath( tszPath, tszName ) );
		if( NULL == hRcDb ) return false;
		m_hRcDb = hRcDb;
		m_uLangID = uLangID;
	}

	return true;
}

void CRcDb::CloseRcDb( void )
{
	if( m_hRcDb != NULL ){
		::FreeLibrary( m_hRcDb );
		m_hRcDb = NULL;
		m_uLangID = 0;
	}

	RemoveAllString();
	RemoveAllFmtMsg();

	return;
}

/*!
	\[Xf[^x[Ẍꗗ쐬B
*/
int CRcDb::LoadRcDbList( RcDbList& rcDbList )
{
	RcDbItem* lpRcDbItem = new RcDbItem;
	lpRcDbItem->m_lpFile = NULL;
	lpRcDbItem->m_lcId = 0;
	lpRcDbItem->m_lpName = new TCHAR[ _tcslen( RCDB_LANG_NAME_DEFAULT ) + 1 ];
	_tcscpy( lpRcDbItem->m_lpName, RCDB_LANG_NAME_DEFAULT );
	rcDbList.push_back( lpRcDbItem );

	HANDLE handle;
	WIN32_FIND_DATA findFileData;

	handle = ::FindFirstFile( RCDB_DLL_NAME_FIND, &findFileData );
	if( INVALID_HANDLE_VALUE == handle ) return 0;

	do{
		TCHAR tszPath[ MAX_PATH ];
		MakeRcDbPath( tszPath, findFileData.cFileName );
		HINSTANCE hRc = ::LoadLibrary( findFileData.cFileName );
		if( NULL != hRc ){
			TCHAR tszName[ 1024 ];
			tszName[ 0 ] = _T('\0');
			::LoadString( hRc, RCDB_IDS_LANG, tszName, sizeof( tszName ) / sizeof( TCHAR ) );
			if( _tcsncmp( tszName, RCDB_LANG_PREFIX, _tcslen( RCDB_LANG_PREFIX ) ) == 0 ){
				LCID lcId = 0;
				TCHAR tszString[ 1024 ];
				tszString[ 0 ] = _T('\0');
				if( 2 == _stscanf( &tszName[ _tcslen( RCDB_LANG_PREFIX ) ], _T("%d,%s"), &lcId, tszString ) ){
					if( lcId != 0 && _tcslen( tszString ) != 0 ){
						lpRcDbItem = new RcDbItem;
						lpRcDbItem->m_lpFile = new TCHAR[ _tcslen( findFileData.cFileName ) + 1 ];
						_tcscpy( lpRcDbItem->m_lpFile, findFileData.cFileName );
						lpRcDbItem->m_lcId = lcId;
						lpRcDbItem->m_lpName = new TCHAR[ _tcslen( tszName ) + 1 ];
						_tcscpy( lpRcDbItem->m_lpName, tszName );
						rcDbList.push_back( lpRcDbItem );
					}
				}
			}
			::FreeLibrary( hRc );
		}
	}while( ::FindNextFile( handle, &findFileData ) );

	::FindClose( handle );

	return (int)rcDbList.size();
}

void CRcDb::FreeRcDbList( RcDbList& rcDbList )
{
	for( int i = 0; i < (int)rcDbList.size(); i++ ){
		if( NULL != rcDbList[ i ]->m_lpFile ) delete [] rcDbList[ i ]->m_lpFile;
		if( NULL != rcDbList[ i ]->m_lpName ) delete [] rcDbList[ i ]->m_lpName;
		delete rcDbList[ i ];
		rcDbList[ i ] = NULL;
	}
	rcDbList.clear();
	return;
}

LPCTSTR CRcDb::GetFormatMessage( DWORD dwID, ... )
{
	RemoveFmtMsg( dwID );

	LPTSTR lpBuffer = NULL;

	va_list argptr;
	va_start( argptr, dwID );

	DWORD dwRet = ::FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_HMODULE,
			GetRcDbInstance(),
			dwID,
			m_uLangID,
			(LPTSTR)&lpBuffer,
			1,
			&argptr
		);

	va_end( argptr );

	if( 0 == dwRet ){
		return NULL;
	}

	m_mapFmtMsg.insert( PairFmtMsg( dwID, lpBuffer ) );

	return lpBuffer;
}

void CRcDb::RemoveFmtMsg( DWORD dwID )
{
	MapFmtMsg::iterator iter = m_mapFmtMsg.find( dwID );
	if( iter == m_mapFmtMsg.end() ) return;
	LPTSTR lp = (LPTSTR)iter->second;
	iter->second = NULL;
	m_mapFmtMsg.erase( iter );
	::LocalFree( lp );
	return;
}

void CRcDb::RemoveAllFmtMsg( void )
{
	MapFmtMsg::iterator iter;
	for( iter = m_mapFmtMsg.begin(); iter != m_mapFmtMsg.end(); iter++ ){
		LPTSTR lp = (LPTSTR)iter->second;
		iter->second = NULL;
		::LocalFree( lp );
	}
	m_mapFmtMsg.clear();
	return;
}

LPCTSTR CRcDb::MakeRcDbPath( LPTSTR lpDst, LPCTSTR lpSrc )
{
	TCHAR tszTmp[ MAX_PATH ];

	DWORD dwRet = ::GetModuleFileName( ::GetModuleHandle( NULL ), tszTmp, sizeof( tszTmp ) / sizeof( TCHAR ) );
	if( 0 == dwRet ){
		_tcscpy( lpDst, lpSrc );
		return lpDst;
	}
	//::PathRemoveFileSpec( tszTmp );
	TCHAR tszDrive[ MAX_PATH ];
	TCHAR tszDir[ MAX_PATH ];
	_tsplitpath( tszTmp, tszDrive, tszDir, NULL, NULL );

	_stprintf( lpDst, _T("%s%s%s"), tszDrive, tszDir, lpSrc );

	return lpDst;
}
