/*!	@file
	@brief Resource Database

	RcDbMessages.h --  Message code definitions for the Sakura-Editor.

	@author wakura
	@date 2008
*/
/*
	Copyright (C) 2008, wakura

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose, 
	including commercial applications, and to alter it and redistribute it 
	freely, subject to the following restrictions:

		1. The origin of this software must not be misrepresented;
		   you must not claim that you wrote the original software.
		   If you use this software in a product, an acknowledgment
		   in the product documentation would be appreciated but is
		   not required.

		2. Altered source versions must be plainly marked as such, 
		   and must not be misrepresented as being the original software.

		3. This notice may not be removed or altered from any source
		   distribution.
*/
#ifndef __RCDB_MESSAGES_H__
#define __RCDB_MESSAGES_H__


// Success      :  9900 -  9999 (0x26AC - 0x270F)
// Informational: 10000 - 10999 (0x2710 - 0x2AF7)
// Warning      : 11000 - 11999 (0x2AF8 - 0x2EDF)
// Error        : 12000 - 12999 (0x2EE0 - 0x32C7)
// Question     : 13000 - 13999 (0x32C8 - 0x36AF)
// Others       : 14000 - 14999 (0x36B0 - 0x3A97)
#define RCDB_MSG_S_BASE  9900
#define RCDB_MSG_I_BASE 10000
#define RCDB_MSG_W_BASE 11000
#define RCDB_MSG_E_BASE 12000
#define RCDB_MSG_Q_BASE 13000
#define RCDB_MSG_O_BASE 14000

//
//  Values are 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|C|R|     Facility          |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  where
//
//      Sev - is the severity code
//
//          00 - Success
//          01 - Informational
//          10 - Warning
//          11 - Error
//
//      C - is the Customer code flag
//
//      R - is a reserved bit
//
//      Facility - is the facility code
//
//      Code - is the facility's status code
//
//
// Define the facility codes
//
#define RCDB_FACILITY_SYSTEM             0x0
#define RCDB_FACILITY_STUBS              0x3
#define RCDB_FACILITY_SECURITY           0x9
#define RCDB_FACILITY_RUNTIME            0x2
#define RCDB_FACILITY_IO_ERROR_CODE      0x4
#define RCDB_FACILITY_INTERNET           0xC


//
// Define the severity codes
//
#define RCDB_STATUS_SEVERITY_WARNING     0x2
#define RCDB_STATUS_SEVERITY_SUCCESS     0x0
#define RCDB_STATUS_SEVERITY_INFORMATIONAL 0x1
#define RCDB_STATUS_SEVERITY_ERROR       0x3


//
// MessageId: RCDB_MSG_S_LANG_INFORMATION
//
// MessageText:
//
//  Sakura-Editor,1041,Japanese
//
#define RCDB_MSG_S_LANG_INFORMATION      ((DWORD)0x0002270FL)

//
// MessageId: RCDB_MSG_I_DELETE_HISTORY
//
// MessageText:
//
//  ŋߎg%1!s!̗폜܂B
//
#define RCDB_MSG_I_DELETE_HISTORY        ((DWORD)0x40022710L)

//
// MessageId: RCDB_MSG_I_DELETE_HISTORY_WITHOUT_FAVORITE
//
// MessageText:
//
//  ŋߎg%1!s!̗(CɓȊO)폜܂B
//
#define RCDB_MSG_I_DELETE_HISTORY_WITHOUT_FAVORITE ((DWORD)0x40022711L)

//
// MessageId: RCDB_MSG_W_SET_SEARCH_CONDITION
//
// MessageText:
//
//  w肵ĂB
//
#define RCDB_MSG_W_SET_SEARCH_CONDITION  ((DWORD)0x80022AF8L)

//
// MessageId: RCDB_MSG_E_CREATE_MUTEX
//
// MessageText:
//
//  CreateMutex() Ɏs܂B%nTNGfB^I܂B
//
#define RCDB_MSG_E_CREATE_MUTEX          ((DWORD)0xC0002EE0L)

//
// MessageId: RCDB_MSG_E_CREATE_EVENT
//
// MessageText:
//
//  CreateEvent() Ɏs܂B%nTNGfB^I܂B
//
#define RCDB_MSG_E_CREATE_EVENT          ((DWORD)0xC0002EE1L)

//
// MessageId: RCDB_MSG_E_CREATE_TARY_WINDOW
//
// MessageText:
//
//  EChE̍쐬Ɏs܂B%nNł܂B
//
#define RCDB_MSG_E_CREATE_TARY_WINDOW    ((DWORD)0xC0022AF9L)

//
// MessageId: RCDB_MSG_E_CREATE_SET_EVENT
//
// MessageText:
//
//  SetEvent() Ɏs܂B%nNł܂B
//
#define RCDB_MSG_E_CREATE_SET_EVENT      ((DWORD)0xC0002EE2L)

//
// MessageId: RCDB_MSG_E_SET_SEARCH_CONDITION
//
// MessageText:
//
//  w肵ĂB
//
#define RCDB_MSG_E_SET_SEARCH_CONDITION  ((DWORD)0xC0022AFAL)

//
// MessageId: RCDB_MSG_E_SET_SEARCH_FOLDER
//
// MessageText:
//
//  ΏۃtH_܂B
//
#define RCDB_MSG_E_SET_SEARCH_FOLDER     ((DWORD)0xC0022AFBL)

//
// MessageId: RCDB_MSG_E_SET_INVALID_FOLDER
//
// MessageText:
//
//  ΏۃtH_w肵ĂB
//
#define RCDB_MSG_E_SET_INVALID_FOLDER    ((DWORD)0xC0022AFCL)

//
// MessageId: RCDB_MSG_E_SET_INVALID_LINE_NUMBER
//
// MessageText:
//
//  sԍ͂ĂB
//
#define RCDB_MSG_E_SET_INVALID_LINE_NUMBER ((DWORD)0xC0022AFDL)

//
// MessageId: RCDB_MSG_Q_DELETE_HISTORY_WITH_FAVORITE
//
// MessageText:
//
//  ŋߎg%1!s!̗폜܂B%n%n낵łH%n%nu͂v\tׂč폜܂B%nuv\tCɓȊO폜܂B
//
#define RCDB_MSG_Q_DELETE_HISTORY_WITH_FAVORITE ((DWORD)0x400232C8L)

//
// MessageId: RCDB_MSG_Q_DELETE_HISTORY
//
// MessageText:
//
//  ŋߎg%1!s!̗폜܂B%n%n낵łH
//
#define RCDB_MSG_Q_DELETE_HISTORY        ((DWORD)0x400232C9L)

//
// MessageId: RCDB_MSG_O_SAMPLE
//
// MessageText:
//
//  ̓TvłB%n(%1!s!%2!d!%3!d!̘ał%.)B
//
#define RCDB_MSG_O_SAMPLE                ((DWORD)0x400236B0L)


#endif	//__RCDB_MESSAGES_H__
